/*
Bump mapping shader by daxnitro.  
This shader implements parallax occlusion and specular mapping and requires terrain_nh.png and terrain_s.png to be included in the current texture pack.
*/

// To disable a particular feature of this shader, add two forward slashes to the beginning its line:

#define ENABLE_PARALLAX_OCCLUSION
#define ENABLE_SPECULAR_MAPPING
#define ENABLE_FOG

/*
For example, to disable fog change the line above to this:

//#define ENABLE_FOG
*/

uniform sampler2D sampler0;
uniform sampler2D sampler1;
uniform sampler2D sampler2;

uniform int renderType;

uniform int fogMode;

varying float specMultiplier;

varying float isBottomFace;

varying vec4 vertColor;
varying vec3 lightVector;
varying vec3 viewVector;

const int GL_LINEAR = 9729;
const int GL_EXP = 2048;

const int RENDER_TYPE_UNKNOWN = 0;

const float MAX_DISTANCE = 100.0;

const int MAX_POINTS = 50;

void main() {
	
	if (gl_FogFragCoord <= MAX_DISTANCE && renderType != RENDER_TYPE_UNKNOWN && viewVector.z < 0.0) {
		
		vec3 coord = vec3(gl_TexCoord[0].st, 1.0);
		
#ifdef ENABLE_PARALLAX_OCCLUSION
		if (texture2D(sampler1, coord.st).a < 1.0) {	
			vec2 minCoord = vec2(gl_TexCoord[0].s - mod(gl_TexCoord[0].s, 0.0625), gl_TexCoord[0].t - mod(gl_TexCoord[0].t, 0.0625));
			vec2 maxCoord = vec2(minCoord.s + 0.0625, minCoord.t + 0.0625);
		
			/* The interval might need to be tweaked per texture pack.  
			   The first two numbers determine how many samples are taken per fragment.
			   The third number divided by one of the first two numbers is inversely proportional to the range of the height-map. */
			   
			vec3 interval = viewVector * vec3(0.0039, 0.0039, 4.5); // Fine for 16x16 tile size
			//vec3 interval = viewVector * vec3(0.0019, 0.0019, 0.5); // Fine for 32x32 tile size
			//vec3 interval = viewVector * vec3(0.00048828125, 0.00048828125, 0.2); // Fine for 128x128 tile size
		
			for (int loopCount = 0; texture2D(sampler1, coord.st).a < coord.z && loopCount < MAX_POINTS; ++loopCount) {
				coord += interval;
				if (coord.s < minCoord.s) {
					coord.s += 0.0625;
				} else if (coord.s >= maxCoord.s) {
					coord.s -= 0.0625;
				}
				if (coord.t < minCoord.t) {
					coord.t += 0.0625;
				} else if (coord.t >= maxCoord.t) {
					coord.t -= 0.0625;
				}
			}
		}
#endif
	
		gl_FragColor = texture2D(sampler0, coord.st) * vertColor;

#ifdef ENABLE_SPECULAR_MAPPING
		if (isBottomFace == 0.0) {
			vec3 bump = texture2D(sampler1, coord.st).xyz * 2.0 - 1.0;
			float specular = clamp(dot(reflect(-lightVector, bump), viewVector), 0.0, 1.0);
			gl_FragColor = clamp(gl_FragColor + texture2D(sampler2, coord.st) * specular * specular * specular * specMultiplier, 0.0, 1.0);
		}
#endif

	} else {

		gl_FragColor = texture2D(sampler0, gl_TexCoord[0].st) * vertColor;

	}
	
#ifdef ENABLE_FOG
	if (gl_FragColor.a != 0.0) {
		if (fogMode == GL_EXP) {
			gl_FragColor = mix(gl_FragColor, gl_Fog.color, 1.0 - clamp(exp(-gl_Fog.density * gl_FogFragCoord), 0.0, 1.0));
		} else if (fogMode == GL_LINEAR) {
			gl_FragColor = mix(gl_FragColor, gl_Fog.color, clamp((gl_FogFragCoord - gl_Fog.start) * gl_Fog.scale, 0.0, 1.0));
		}
	}
#endif
	
}