/*
Bump mapping shader by daxnitro.  
This shader implements parallax occlusion and specular mapping and requires terrain_nh.png and terrain_s.png to be included in the current texture pack.
*/

// To enable world curvature, remove the slashes from the next line.

//#define ENABLE_WORLD_CURVATURE

varying vec4 vertColor;
varying vec3 viewVector;
varying vec3 lightVector;

uniform vec3 sunVector;
uniform vec3 moonVector;

varying float specMultiplier;

uniform int worldTime;

uniform int renderType;

const int RENDER_TYPE_UNKNOWN = 0;

varying float isBottomFace;

#ifdef ENABLE_WORLD_CURVATURE
	const float WORLD_RADIUS         = 2500.0;
	const float WORLD_RADIUS_SQUARED = 6250000.0;
#endif

void main() {
	
	#ifdef ENABLE_WORLD_CURVATURE
		vec4 position = gl_ModelViewMatrix * gl_Vertex;
		
		if (gl_Color.a != 0.8) {
			// Not a cloud.
			
			float distanceSquared = position.x * position.x + position.z * position.z;
			
			position.y -= WORLD_RADIUS - sqrt(max(1.0 - distanceSquared / WORLD_RADIUS_SQUARED, 0.0)) * WORLD_RADIUS;
		}
		
		gl_Position = gl_ProjectionMatrix * position;
	#else
		gl_Position = ftransform();
	#endif

	vertColor = gl_Color;

	gl_TexCoord[0] = gl_MultiTexCoord0;
	
	if (renderType != RENDER_TYPE_UNKNOWN) {
		vec3 normal = normalize(gl_NormalMatrix * gl_Normal);
		vec3 tangent;
		vec3 binormal;
		
		isBottomFace = 0.0;
	
		if (gl_Normal.x == 1.0) {
			//  1.0,  0.0,  0.0
			tangent  = normalize(gl_NormalMatrix * vec3( 0.0,  0.0, -1.0));
			binormal = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
		} else if (gl_Normal.x < 0.0) {
			// -1.0,  0.0,  0.0
			tangent  = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
			binormal = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
		} else if (gl_Normal.y == 1.0) {
			//  0.0,  1.0,  0.0
			tangent  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
			binormal = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
		} else if (gl_Normal.y < 0.0) {
			//  0.0, -1.0,  0.0
			isBottomFace = 1.0;
			tangent  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
			binormal = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
		} else if (gl_Normal.z == 1.0) {
			//  0.0,  0.0,  1.0
			tangent  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
			binormal = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
		} else if (gl_Normal.z < 0.0) {
			//  0.0,  0.0, -1.0
			tangent  = normalize(gl_NormalMatrix * vec3(-1.0,  0.0,  0.0));
			binormal = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
		}
		
		mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
	                          tangent.y, binormal.y, normal.y,
	                          tangent.z, binormal.z, normal.z);
		
		viewVector = (gl_ModelViewMatrix * gl_Vertex).xyz;
		viewVector = normalize(tbnMatrix * viewVector);

		if (worldTime < 12500 || worldTime > 23000) {
			lightVector = normalize(tbnMatrix * -sunVector);
			specMultiplier = 1.0;
		} else {
			lightVector = normalize(tbnMatrix * -moonVector);
			specMultiplier = 0.5;
		}
	}
	
	gl_FogFragCoord = gl_Position.z;
}